package cn.com.qmth.scancentral.service.impl;

import cn.com.qmth.scancentral.dao.AssignedCheckHistoryDao;
import cn.com.qmth.scancentral.entity.AssignedCheckHistoryEntity;
import cn.com.qmth.scancentral.service.AssignedCheckHistoryService;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.boot.core.exception.ParameterException;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

@Service
public class AssignedCheckHistoryServiceImpl extends ServiceImpl<AssignedCheckHistoryDao, AssignedCheckHistoryEntity>
        implements AssignedCheckHistoryService {

    @Transactional
    @Override
    public void deleteByStudentId(Long studentId) {
        if (studentId == null) {
            throw new ParameterException("studentId 不能为空");
        }
        UpdateWrapper<AssignedCheckHistoryEntity> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().eq(AssignedCheckHistoryEntity::getStudentId, studentId);
        this.baseMapper.delete(updateWrapper);
    }

    @Override
    @Transactional
    public void save(Long userId, Long studentId, Long examId) {
        if (studentId == null) {
            throw new ParameterException("studentId 不能为空");
        }
        if (userId == null) {
            throw new ParameterException("userId 不能为空");
        }
        AssignedCheckHistoryEntity entity = this.findByStudentIdAndUserId(studentId, userId);
        if (entity == null) {
            entity = new AssignedCheckHistoryEntity(userId, studentId, examId);
        }
        this.saveOrUpdate(entity);
    }

    @Override
    public AssignedCheckHistoryEntity findByStudentIdAndUserId(Long studentId, Long userId) {
        QueryWrapper<AssignedCheckHistoryEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<AssignedCheckHistoryEntity> lw = wrapper.lambda();
        lw.eq(AssignedCheckHistoryEntity::getStudentId, studentId);
        lw.eq(AssignedCheckHistoryEntity::getUserId, userId);
        return this.getOne(wrapper);
    }

    @Override
    public int getCountByUserId(Long userId, Long examId) {
        QueryWrapper<AssignedCheckHistoryEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<AssignedCheckHistoryEntity> lw = wrapper.lambda();
        lw.eq(AssignedCheckHistoryEntity::getUserId, userId);
        lw.eq(AssignedCheckHistoryEntity::getExamId, examId);
        return this.count(wrapper);
    }
}
