package cn.com.qmth.scancentral.service.impl;

import cn.com.qmth.scancentral.bean.SchoolSession;
import cn.com.qmth.scancentral.consumer.UploadConsumer;
import cn.com.qmth.scancentral.enums.SystemMode;
import cn.com.qmth.scancentral.service.SessionService;
import cn.com.qmth.scancentral.service.StudentService;
import cn.com.qmth.scancentral.service.UploadService;
import cn.com.qmth.scancentral.support.SpringContextHolder;
import cn.com.qmth.scancentral.vo.StudentUploadVo;
import org.apache.commons.collections4.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Collection;
import java.util.List;
import java.util.concurrent.*;

@Service
public class UploadServiceImpl implements UploadService {

    private static final Logger log = LoggerFactory.getLogger(UploadService.class);

    @Autowired
    private SessionService sessionService;

    @Autowired
    private StudentService studentService;

    @Override
    public void upload() {
        if (!SystemMode.MARKINGCLOUD.equals(SystemMode.current())) {
            return;
        }
        ExecutorService ex = null;
        try {
            Collection<SchoolSession> sss = sessionService.getSchoolSessions();
            if (CollectionUtils.isEmpty(sss)) {
                return;
            }
            int count = Math.min(5, Runtime.getRuntime().availableProcessors() * 2);
            ex = getExecutor(count);
            for (SchoolSession ss : sss) {
                if (studentService.existUploadData(ss.getId())) {
                    for (;;) {
                        List<StudentUploadVo> dtos = findOneBatchData(ss.getId());
                        if (CollectionUtils.isEmpty(dtos)) {
                            // 拿不到数据，结束
                            break;
                        }
                        // 开始处理一轮
                        disposeBatch(ex, dtos);
                    }
                }
            }
        } catch (Exception e) {
            log.error("上传任务出错", e);
        } finally {
            if (ex != null) {
                ex.shutdown();
            }
        }
    }

    private void disposeBatch(ExecutorService ex, List<StudentUploadVo> dtos) throws InterruptedException {
        CountDownLatch endGate = new CountDownLatch(dtos.size());
        for (StudentUploadVo vo : dtos) {
            UploadConsumer com = SpringContextHolder.getBean(UploadConsumer.class);
            com.setVo(vo);
            com.setEndGate(endGate);
            ex.execute(com);
        }
        endGate.await();
    }

    private List<StudentUploadVo> findOneBatchData(Long schoolId) {
        if (sessionService.getSchoolSession(schoolId) == null) {
            return null;
        }
        return studentService.findToUpload(200, schoolId);
    }

    private ExecutorService getExecutor(int threadCount) {
        ExecutorService exec = new ThreadPoolExecutor(threadCount, threadCount, 0L, TimeUnit.SECONDS,
                new LinkedBlockingQueue<>(threadCount * 2), r -> {
                    Thread t = new Thread(r);
                    return t;
                }, (r, executor) -> {
                    if (!executor.isShutdown()) {
                        try {
                            executor.getQueue().put(r);
                        } catch (InterruptedException e) {
                            throw new RuntimeException(e);
                        }
                    }
                });
        return exec;
    }
}
